// DFT.H Defines DFTobject class
// Copyright 2006 Chad Berchek

// See revision history in dft.cpp file


#ifndef DFT_H
#define DFT_H

#include "wavetypes.h"
using namespace std;

class DFTobject {
	public:
		DFTobject(const samplingRateT setSamplingRate, const sampleT * const dataPtr = 0);
		void setFreq(frequencyT freq);
		void setLen(sampleCountT n);
		void setSamplingRate(samplingRateT samplingRate);
		void setFreqAndLen(frequencyT freq, sampleCountT len);
		void setNewData(const sampleT * const dataPtr);

		amplitudeT getAmplitude();
		phaseT getPhase();
		amplitudeT getRawAmplitude();
		phaseT getRawPhase();

		void invalidate();

	private:
		frequencyT frequency;
		sampleCountT len;
		const sampleT* sampleData;
		samplingRateT samplingRate;
	
		struct DFTstate {
			amplitudeT amplitude;
			phaseT phase;
			bool isCurrent;
		};
	
		DFTstate raw;
		
		void Dft();
};

// Extra functions which are for getting a good DFT frame size
double getFractionOfCycle(const double cycles);
unsigned int getNumSamplesForDFT(const double freq, const unsigned int samplingRate, const double toneDuration,
	const double maxFractionalCycle, const unsigned int minNumCycles, const unsigned int maxExtraSamples);

#endif
